Package: ClusterFoldSimilarity
Type: Package
Title: Calculate similarity of clusters from different single cell
        samples using foldchanges
Version: 1.8.0
Authors@R: person("Oscar", "Gonzalez-Velasco",
  email = "oscargvelasco@gmail.com",
  role = c("cre", "aut"),
  comment = c(ORCID = "0000-0002-5054-8635"))
Description: This package calculates a similarity coefficient using the fold changes of shared features (e.g. genes) among clusters of different samples/batches/datasets. The similarity coefficient is calculated using the dot-product (Hadamard product) of every pairwise combination of Fold Changes between a source cluster i of sample/dataset n and all the target clusters j in sample/dataset m
License: Artistic-2.0
Encoding: UTF-8
Imports: methods, igraph, ggplot2, scales, BiocParallel, graphics,
        stats, utils, Matrix, cowplot, dplyr, reshape2, Seurat,
        SeuratObject, SingleCellExperiment, ggdendro
Suggests: knitr, rmarkdown, kableExtra, scRNAseq, BiocStyle
RoxygenNote: 7.2.3
biocViews: SingleCell, Clustering, FeatureExtraction, GraphAndNetwork,
        GeneTarget, RNASeq
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/ClusterFoldSimilarity
git_branch: RELEASE_3_23
git_last_commit: ebc46cd
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 21:24:44 UTC; biocbuild
Author: Oscar Gonzalez-Velasco [cre, aut] (ORCID:
    <https://orcid.org/0000-0002-5054-8635>)
Maintainer: Oscar Gonzalez-Velasco <oscargvelasco@gmail.com>
