# Generated by roxygen2: do not edit by hand

export(clusterFoldSimilarity)
export(findCommunitiesSimmilarity)
export(plotClustersGraph)
export(similarityHeatmap)
import(Seurat)
import(ggplot2)
importFrom(BiocParallel,bplapply)
importFrom(Matrix,Matrix)
importFrom(Matrix,rowMeans)
importFrom(SingleCellExperiment,colLabels)
importFrom(SingleCellExperiment,counts)
importFrom(cowplot,align_plots)
importFrom(cowplot,plot_grid)
importFrom(dplyr,"%>%")
importFrom(dplyr,arrange)
importFrom(dplyr,filter)
importFrom(ggdendro,ggdendrogram)
importFrom(graphics,legend)
importFrom(graphics,par)
importFrom(igraph,V)
importFrom(igraph,cluster_infomap)
importFrom(igraph,graph_from_data_frame)
importFrom(igraph,layout_with_fr)
importFrom(igraph,membership)
importFrom(igraph,modularity)
importFrom(methods,is)
importFrom(reshape2,melt)
importFrom(scales,alpha)
importFrom(stats,dist)
importFrom(stats,hclust)
importFrom(stats,median)
importFrom(stats,optim)
importFrom(stats,pnorm)
importFrom(stats,quantile)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,var)
importFrom(utils,head)
